/* The a way to make the different game levels would be to extract the parameters
   from ActionState and create a Level data structure or class...
	 due to time constraints, I simply replicated ActionState and changed what was
	 necessary for each game level.
*/

function TutorialState() {
	this.setup = function() {
		// Time counter for advancing the command queue
		this.dt = 0;

		// Flags indicating that players have pressed the F1 or F8 keys
		this.p1act = false;
		this.p2act = false;

		// Command queue, filled with fixed starting commands
		this.p1q = [1, 3, 5, 1];
		this.p2q = [7, 5, 3, 7];

		// Have the robots reached their destinations?
		this.p1ok = false;
		this.p2ok = false;

		// Loads up graphic resources and set up sprites
		this.baseImg = new jaws.Sprite({image:"base.png", x:0, y:0});
		this.p1 = new jaws.Sprite({image:"robot_red.png", x:GlobalData.MINX, y:GlobalData.MINY+3*GlobalData.BLOCK});
		this.p2 = new jaws.Sprite({image:"robot_blue.png", x:GlobalData.MAXX, y:GlobalData.MINY+5*GlobalData.BLOCK});
		this.arrows = new jaws.SpriteSheet({image:"arrows.png", frame_size:[16, 16]});
		this.beacons = new jaws.Animation({sprite_sheet:"beacons.png", frame_size:[32, 32], frame_duration:250});
		this.b1 = new jaws.Sprite({x: GlobalData.MINX + 9*GlobalData.BLOCK, y: GlobalData.MINY + 3*GlobalData.BLOCK});
		this.b1.anim_default = this.beacons.slice(0, 3);
		this.b1.anim_default.bounce = true;
		this.b2 = new jaws.Sprite({x: GlobalData.MINX + 9*GlobalData.BLOCK, y: GlobalData.MINY + 5*GlobalData.BLOCK});
		this.b2.anim_default = this.beacons.slice(3, 6);
		this.b2.anim_default.bounce = true;

		// And audio, too
		this.bgm = new Howl({src: ['bgm.ogg', 'bgm.mp3'], volume:0.6, loop:true});
		this.blip = new Howl({src: ['walk.ogg', 'walk.mp3'], volume:0.8});
		this.blop = new Howl({src: ['power.ogg', 'power.mp3'], volume:0.8});

		// Set up keyboard handlers
		theState = this;
		jaws.on_keydown("f1", function() {theState.p1act = true;});
		jaws.on_keydown("f8", function() {theState.p2act = true;});

		// Startup message
		this.msg = new jaws.Text({x:320, y:180, alpha:1, text:"MEET",
			fontFace:"sans-serif", fontSize:80, textAlign:"center", textBaseline:"middle",
			color:"#F4F9CA"});
		this.bgm.play();
	}

	// Performs a move action depending on a given command code
	this.doMove = function(v) {
		switch(v) {
			case 0:
				if (this.p1.y > GlobalData.MINY)
					this.p1.y -= GlobalData.BLOCK;
				break;
			case 1:
				if (this.p1.x < GlobalData.MAXX)
					this.p1.x += GlobalData.BLOCK;
				break;
			case 2:
				if (this.p1.y < GlobalData.MAXY)
					this.p1.y += GlobalData.BLOCK;
				break;
			case 3:
				if (this.p1.x > GlobalData.MINX)
					this.p1.x -= GlobalData.BLOCK;
				break;
			case 4:
				if (this.p2.y > GlobalData.MINY)
					this.p2.y -= GlobalData.BLOCK;
				break;
			case 5:
				if (this.p2.x < GlobalData.MAXX)
					this.p2.x += GlobalData.BLOCK;
				break;
			case 6:
				if (this.p2.y < GlobalData.MAXY)
					this.p2.y += GlobalData.BLOCK;
				break;
			case 7:
				if (this.p2.x > GlobalData.MINX)
					this.p2.x -= GlobalData.BLOCK;
				break;
		}
	}

	// Advance the command queues, adding a new command to the end
	this.moveQueues = function() {
		var i = 0;
		while (i < 3) {
			this.p1q[i] = this.p1q[i+1];
			this.p2q[i] = this.p2q[i+1];
			i++;
		}
		this.p1q[3] = 1+2*Math.floor(Math.random()*4);
		do
			this.p2q[3] = 1+2*Math.floor(Math.random()*4);
		while (this.p2q[3] == this.p1q[3]);
	}

	// Check for robot-goal contact
	this.checkContact = function() {
		if (this.p1.x == this.b1.x && this.p1.y == this.b1.y) {
			if (!this.p1ok) {
				this.p1ok = true;
				this.blop.play();
			}
		}
		else
			this.p1ok = false;

		if (this.p2.x == this.b2.x && this.p2.y == this.b2.y) {
			if (!this.p2ok) {
				this.p2ok = true;
				this.blop.play();
			}
		}
		else
			this.p2ok = false;
	}

	// Good ol' game simulation
	this.update = function() {
		// Update the queue
    this.dt += jaws.game_loop.tick_duration;
		if (this.dt >= GlobalData.DELAY) {
			this.dt -= GlobalData.DELAY;
			this.moveQueues();
		}

		// Move robots if requested
		if (this.p1act) {
			this.doMove(this.p1q[0]);
			this.blip.play();
			this.p1act = false;
		}
		if (this.p2act) {
			this.doMove(this.p2q[0]);
			this.blip.play();
			this.p2act = false;
		}

		// Text message effect
		if (this.msg.alpha > 0) {
			this.msg.alpha -= 0.01;
		}

		// Check for victory
		this.checkContact();
		if (this.p1ok && this.p2ok) {
			this.bgm.stop();
			GlobalData.targetState = ActionState;
			jaws.switchGameState(Transition);
		}
	}

	// Draw the command queues on screen
	this.drawQueues = function() {
		var i = 0;
		for (i = 0; i < 4; i++) {
			jaws.context.drawImage(this.arrows.frames[this.p1q[i]], 276 - 40*i, 24);
			jaws.context.drawImage(this.arrows.frames[this.p2q[i]], 348 + 40*i, 24);
  	}
	}

	// Good ol' drawing callback
	this.draw = function() {
		this.baseImg.draw();

		this.b1.setImage(this.b1.anim_default.next());
		this.b1.draw();
		this.b2.setImage(this.b2.anim_default.next());
		this.b2.draw();

		this.p1.draw();
		this.p2.draw();

		this.drawQueues();

		if (this.msg.alpha > 0)
			this.msg.draw();
	}
}
